@@ -398,7 +398,7 @@ flatpak_builtin_add_remote (int argc, char **argv,
     }
 
   if (opt_from ||
-      g_str_has_suffix (location, ".flatpakrepo"))
+      flatpak_file_arg_has_suffix (location, ".flatpakrepo"))
     {
       load_options (location, &gpg_data);
       if (opt_url == NULL)
