@@ -33,6 +33,7 @@
 
 #include "flatpak-builtins.h"
 #include "flatpak-utils.h"
+#include "flatpak-error.h"
 #include "flatpak-dbus.h"
 #include "flatpak-run.h"
 
@@ -63,10 +64,16 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
   g_autoptr(GOptionContext) context = NULL;
   g_autoptr(FlatpakDeploy) app_deploy = NULL;
   g_autofree char *app_ref = NULL;
-  char *app;
+  g_autofree char *runtime_ref = NULL;
+  const char *pref;
   int i;
   int rest_argv_start, rest_argc;
   g_autoptr(FlatpakContext) arg_context = NULL;
+  g_autofree char *id = NULL;
+  g_autofree char *arch = NULL;
+  g_autofree char *branch = NULL;
+  FlatpakKinds kinds;
+  g_autoptr(GError) local_error = NULL;
 
   context = g_option_context_new (_("APP [args...] - Run an app"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
@@ -93,33 +100,89 @@ flatpak_builtin_run (int argc, char **argv, GCancellable *cancellable, GError **
   if (rest_argc == 0)
     return usage_error (context, _("APP must be specified"), error);
 
-  app = argv[rest_argv_start];
+  pref = argv[rest_argv_start];
 
-  if (!flatpak_split_partial_ref_arg (app, &opt_arch, &opt_branch, error))
+  if (!flatpak_split_partial_ref_arg (pref, FLATPAK_KINDS_APP | FLATPAK_KINDS_RUNTIME,
+                                      opt_arch, opt_branch,
+                                      &kinds, &id, &arch, &branch, error))
     return FALSE;
 
-  if (opt_branch == NULL && opt_arch == NULL)
+  if (branch == NULL || arch == NULL)
     {
+      g_autofree char *current_ref = NULL;
       g_autoptr(FlatpakDir) user_dir = flatpak_dir_get_user ();
-      g_autoptr(FlatpakDir) system_dir = flatpak_dir_get_system ();
 
-      app_ref = flatpak_dir_current_ref (user_dir, app, cancellable);
-      if (app_ref == NULL)
-        app_ref = flatpak_dir_current_ref (system_dir, app, cancellable);
+      current_ref = flatpak_dir_current_ref (user_dir, id, cancellable);
+      if (current_ref == NULL)
+        {
+          g_autoptr(GPtrArray) system_dirs = NULL;
+
+          system_dirs = flatpak_dir_get_system_list (cancellable, error);
+          if (system_dirs == NULL)
+            return FALSE;
+
+          for (i = 0; i < system_dirs->len; i++)
+            {
+              FlatpakDir *dir = g_ptr_array_index (system_dirs, i);
+              current_ref = flatpak_dir_current_ref (dir, id, cancellable);
+              if (current_ref != NULL)
+                break;
+            }
+        }
+
+      if (current_ref)
+        {
+          g_auto(GStrv) parts = flatpak_decompose_ref (current_ref, NULL);
+          if (parts)
+            {
+              if (branch == NULL)
+                branch = g_strdup (parts[3]);
+              if (arch == NULL)
+                arch = g_strdup (parts[2]);
+            }
+        }
     }
 
-  if (app_ref == NULL)
+  if ((kinds & FLATPAK_KINDS_APP) != 0)
     {
-      app_ref = flatpak_compose_ref (TRUE, app, opt_branch, opt_arch, error);
+      app_ref = flatpak_compose_ref (TRUE, id, branch, arch, &local_error);
       if (app_ref == NULL)
         return FALSE;
+
+      app_deploy = flatpak_find_deploy_for_ref (app_ref, cancellable, &local_error);
+      if (app_deploy == NULL &&
+          (!g_error_matches (local_error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_INSTALLED) ||
+           (kinds & FLATPAK_KINDS_RUNTIME) == 0))
+        {
+          g_propagate_error (error, g_steal_pointer (&local_error));
+          return FALSE;
+        }
+      /* On error, local_error is set after this point so we can reuse
+         this error rather than later errors, as the app-kind error
+         is more likely interesting. */
     }
 
-  app_deploy = flatpak_find_deploy_for_ref (app_ref, cancellable, error);
   if (app_deploy == NULL)
-    return FALSE;
+    {
+      g_autoptr(FlatpakDeploy) runtime_deploy = NULL;
 
-  if (!flatpak_run_app (app_ref, app_deploy,
+      runtime_ref = flatpak_compose_ref (FALSE, id, branch, arch, error);
+      if (runtime_ref == NULL)
+        return FALSE;
+
+      runtime_deploy = flatpak_find_deploy_for_ref (runtime_ref, cancellable, NULL);
+      if (runtime_deploy == NULL)
+        {
+          /* Report old app-kind error, as its more likely right */
+          g_propagate_error (error, g_steal_pointer (&local_error));
+          return FALSE;
+        }
+      /* Clear app-kind error */
+      g_clear_error (&local_error);
+    }
+
+  if (!flatpak_run_app (app_deploy ? app_ref : runtime_ref,
+                        app_deploy,
                         arg_context,
                         opt_runtime,
                         opt_runtime_version,
@@ -142,9 +205,9 @@ flatpak_complete_run (FlatpakCompletion *completion)
 {
   g_autoptr(GOptionContext) context = NULL;
   g_autoptr(FlatpakDir) user_dir = NULL;
-  g_autoptr(FlatpakDir) system_dir = NULL;
+  g_autoptr(GPtrArray) system_dirs = NULL;
   g_autoptr(GError) error = NULL;
-  int i;
+  int i, j;
   g_autoptr(FlatpakContext) arg_context = NULL;
 
   context = g_option_context_new ("");
@@ -167,7 +230,7 @@ flatpak_complete_run (FlatpakCompletion *completion)
       user_dir = flatpak_dir_get_user ();
       {
         g_auto(GStrv) refs = flatpak_dir_find_installed_refs (user_dir, NULL, NULL, opt_arch,
-                                                              TRUE, FALSE, &error);
+                                                              FLATPAK_KINDS_APP, &error);
         if (refs == NULL)
           flatpak_completion_debug ("find local refs error: %s", error->message);
         for (i = 0; refs != NULL && refs[i] != NULL; i++)
@@ -178,20 +241,27 @@ flatpak_complete_run (FlatpakCompletion *completion)
           }
       }
 
-      system_dir = flatpak_dir_get_system ();
-      {
-        g_auto(GStrv) refs = flatpak_dir_find_installed_refs (system_dir, NULL, NULL, opt_arch,
-                                                              TRUE, FALSE, &error);
-        if (refs == NULL)
-          flatpak_completion_debug ("find local refs error: %s", error->message);
-        for (i = 0; refs != NULL && refs[i] != NULL; i++)
-          {
-            g_auto(GStrv) parts = flatpak_decompose_ref (refs[i], NULL);
-            if (parts)
-              flatpak_complete_word (completion, "%s ", parts[1]);
-          }
+      system_dirs = flatpak_dir_get_system_list (NULL, &error);
+      if (system_dirs == NULL) {
+        flatpak_completion_debug ("find system installations error: %s", error->message);
+        break;
       }
 
+      for (i = 0; i < system_dirs->len; i++)
+        {
+          FlatpakDir *dir = g_ptr_array_index (system_dirs, i);
+          g_auto(GStrv) refs = flatpak_dir_find_installed_refs (dir, NULL, NULL, opt_arch,
+                                                                FLATPAK_KINDS_APP, &error);
+          if (refs == NULL)
+            flatpak_completion_debug ("find local refs error: %s", error->message);
+          for (j = 0; refs != NULL && refs[j] != NULL; j++)
+            {
+              g_auto(GStrv) parts = flatpak_decompose_ref (refs[j], NULL);
+              if (parts)
+                flatpak_complete_word (completion, "%s ", parts[1]);
+            }
+        }
+
       break;
 
     }
