@@ -38,6 +38,7 @@ static gboolean opt_ostree_verbose;
 static gboolean opt_version;
 static gboolean opt_default_arch;
 static gboolean opt_supported_arches;
+static gboolean opt_gl_drivers;
 static gboolean opt_user;
 static char *opt_installation;
 
@@ -110,6 +111,7 @@ static GOptionEntry empty_entries[] = {
   { "version", 0, 0, G_OPTION_ARG_NONE, &opt_version, N_("Print version information and exit"), NULL },
   { "default-arch", 0, 0, G_OPTION_ARG_NONE, &opt_default_arch, N_("Print default arch and exit"), NULL },
   { "supported-arches", 0, 0, G_OPTION_ARG_NONE, &opt_supported_arches, N_("Print supported arches and exit"), NULL },
+  { "gl-drivers", 0, 0, G_OPTION_ARG_NONE, &opt_gl_drivers, N_("Print active gl drivers and exit"), NULL },
   { NULL }
 };
 
@@ -236,6 +238,15 @@ flatpak_option_context_parse (GOptionContext     *context,
       exit (EXIT_SUCCESS);
     }
 
+  if (opt_gl_drivers)
+    {
+      const char **drivers = flatpak_get_gl_drivers ();
+      int i;
+      for (i = 0; drivers[i] != NULL; i++)
+        g_print ("%s\n", drivers[i]);
+      exit (EXIT_SUCCESS);
+    }
+
   if (!(flags & FLATPAK_BUILTIN_FLAG_NO_DIR))
     {
       if (opt_user)
