@@ -192,6 +192,12 @@ is_elf_file (const char *path,
   if (!S_ISREG (stbuf.st_mode))
     return FALSE;
 
+  /* Self-extracting .zip files can be ELF-executables, but shouldn't be
+     treated like them - for example, stripping them breaks their
+     operation */
+  if (g_str_has_suffix (filename, ".zip"))
+    return FALSE;
+
   if ((strstr (filename, ".so.") != NULL ||
        g_str_has_suffix (filename, ".so")) ||
       (stbuf.st_mode & 0111) != 0)
@@ -267,11 +273,8 @@ migrate_locale_dir (GFile      *source_dir,
   while ((next = g_file_enumerator_next_file (dir_enum, NULL, &temp_error)))
     {
       g_autoptr(GFileInfo) child_info = next;
-      g_autoptr(GFile) child = NULL;
       g_autoptr(GFile) locale_subdir = NULL;
 
-      child = g_file_get_child (source_dir, g_file_info_get_name (child_info));
-
       if (g_file_info_get_file_type (child_info) == G_FILE_TYPE_DIRECTORY)
         {
           g_autoptr(GFile) child = NULL;
