@@ -98,6 +98,7 @@ typedef enum {
   FLATPAK_PULL_FLAGS_NONE = 0,
   FLATPAK_PULL_FLAGS_DOWNLOAD_EXTRA_DATA = 1 << 0,
   FLATPAK_PULL_FLAGS_SIDELOAD_EXTRA_DATA = 1 << 1,
+  FLATPAK_PULL_FLAGS_ALLOW_DOWNGRADE = 1 << 2,
 } FlatpakPullFlags;
 
 typedef enum {
@@ -451,6 +452,8 @@ gboolean    flatpak_dir_collect_unmaintained_refs (FlatpakDir   *self,
                                                    GHashTable   *hash,
                                                    GCancellable *cancellable,
                                                    GError      **error);
+gboolean   flatpak_dir_remote_has_deploys (FlatpakDir   *self,
+                                           const char   *remote);
 char      *flatpak_dir_create_origin_remote (FlatpakDir   *self,
                                              const char   *url,
                                              const char   *id,
