@@ -182,7 +182,7 @@ flatpak_get_supported_arches (void)
  * Lists the system installations according to the current configuration and current
  * availability (e.g. doesn't return a configured installation if not reachable).
  *
- * Returns: (transfer full) (element-type FlatpakInstallation): an GPtrArray of
+ * Returns: (transfer container) (element-type FlatpakInstallation): an GPtrArray of
  *   #FlatpakInstallation instances
  *
  * Since: 0.8
@@ -200,7 +200,7 @@ flatpak_get_system_installations (GCancellable *cancellable,
   if (system_dirs == NULL)
     goto out;
 
-  installs = g_ptr_array_new ();
+  installs = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
   for (i = 0; i < system_dirs->len; i++)
     {
       g_autoptr(GError) local_error = NULL;
@@ -844,7 +844,7 @@ flatpak_installation_list_installed_refs_for_update (FlatpakInstallation *self,
         }
       else
         {
-          g_debug ("Update: Failed to read remote %s: %s\n",
+          g_debug ("Update: Failed to read remote %s: %s",
                    flatpak_remote_get_name (remote),
                    local_error->message);
         }
@@ -1097,6 +1097,30 @@ flatpak_installation_load_app_overrides (FlatpakInstallation *self,
   return metadata_contents;
 }
 
+static inline guint
+get_metadata_progress (guint metadata_fetched,
+                       guint outstanding_metadata_fetches)
+{
+  guint total_metadata = metadata_fetched + outstanding_metadata_fetches;
+
+  /* Defensive check */
+  if (total_metadata == 0)
+    return 1;
+
+  /* Below 5, there's still a high chance to overestimate the
+   * progress */
+  if (total_metadata < 5)
+    return 1;
+
+  return (guint) 10 * (metadata_fetched / (gdouble) total_metadata);
+}
+
+static inline guint
+get_write_progress (guint outstanding_writes)
+{
+  return (guint) (3 / (gdouble) outstanding_writes);
+}
+
 static void
 progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
 {
@@ -1112,13 +1136,27 @@ progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
   guint total_delta_parts;
   guint64 bytes_transferred;
   guint64 total_delta_part_size;
+  guint outstanding_extra_data;
+  guint64 total_extra_data_bytes;
+  guint64 transferred_extra_data_bytes;
   guint fetched;
   guint metadata_fetched;
   guint requested;
-  guint64 elapsed_time;
+  guint64 current_time;
   guint new_progress = 0;
   gboolean estimating = FALSE;
 
+  /* The heuristic here goes as follows:
+   *  - If we have delta files, grow up to 45%
+   *  - Else:
+   *     - While fetching metadata, grow up to 10%
+   *     - Then, while fetch the file objects, grow up to:
+   *        - 45% if we have extra data to download
+   *        - 90% otherwise
+   * - Writing the objects goes from 52% to 55%, or 97% to 100%
+   * - Extra data, if present, will go from 55% to 100%
+   */
+
   buf = g_string_new ("");
 
   status = ostree_async_progress_get_status (progress);
@@ -1130,18 +1168,26 @@ progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
   total_delta_parts = ostree_async_progress_get_uint (progress, "total-delta-parts");
   total_delta_part_size = ostree_async_progress_get_uint64 (progress, "total-delta-part-size");
   bytes_transferred = ostree_async_progress_get_uint64 (progress, "bytes-transferred");
+  outstanding_extra_data = ostree_async_progress_get_uint (progress, "outstanding-extra-data");
+  total_extra_data_bytes = ostree_async_progress_get_uint64 (progress, "total-extra-data-bytes");
+  transferred_extra_data_bytes = ostree_async_progress_get_uint64 (progress, "transferred-extra-data-bytes");
   fetched = ostree_async_progress_get_uint (progress, "fetched");
   metadata_fetched = ostree_async_progress_get_uint (progress, "metadata-fetched");
   requested = ostree_async_progress_get_uint (progress, "requested");
-  elapsed_time = (g_get_monotonic_time () - ostree_async_progress_get_uint64 (progress, "start-time")) / G_USEC_PER_SEC;
+  current_time = g_get_monotonic_time ();
 
   if (status)
     {
       g_string_append (buf, status);
+
+      /* The status is sent on error or when the pull is finished */
+      new_progress = outstanding_extra_data ? 55 : 100;
     }
   else if (outstanding_fetches)
     {
-      guint64 bytes_sec = bytes_transferred / elapsed_time;
+      guint64 elapsed_time =
+        (current_time - ostree_async_progress_get_uint64 (progress, "start-time")) / G_USEC_PER_SEC;
+      guint64 bytes_sec = (elapsed_time > 0) ? bytes_transferred / elapsed_time : 0;
       g_autofree char *formatted_bytes_transferred =
         g_format_size_full (bytes_transferred, 0);
       g_autofree char *formatted_bytes_sec = NULL;
@@ -1160,7 +1206,12 @@ progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
                                   formatted_bytes_sec, formatted_bytes_transferred,
                                   formatted_total);
 
-          new_progress = (100 * bytes_transferred) / total_delta_part_size;
+
+          /* When we have delta files, no metadata is fetched */
+          if (outstanding_extra_data > 0)
+            new_progress = (52 * bytes_transferred) / total_delta_part_size;
+          else
+            new_progress = (97 * bytes_transferred) / total_delta_part_size;
         }
       else if (outstanding_metadata_fetches)
         {
@@ -1168,23 +1219,55 @@ progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
            * Since its really hard to figure out early how much data there is we report 1% until
            * all objects are scanned. */
 
-          new_progress = 1;
           estimating = TRUE;
 
           g_string_append_printf (buf, "Receiving metadata objects: %u/(estimating) %s/s %s",
                                   metadata_fetched, formatted_bytes_sec, formatted_bytes_transferred);
+
+          /* Go up to 10% until the metadata is all fetched */
+          new_progress = get_metadata_progress (metadata_fetched, outstanding_metadata_fetches);
         }
       else
         {
-          new_progress = (100 * fetched) / requested;
           g_string_append_printf (buf, "Receiving objects: %u%% (%u/%u) %s/s %s",
                                   (guint) ((((double) fetched) / requested) * 100),
                                   fetched, requested, formatted_bytes_sec, formatted_bytes_transferred);
+
+
+          if (outstanding_extra_data)
+            new_progress = 10 + (42 * (double) fetched) / requested;
+          else
+            new_progress = 10 + (87 * (double) fetched) / requested;
         }
     }
+  else if (outstanding_extra_data)
+    {
+      guint64 elapsed_time =
+        (current_time - ostree_async_progress_get_uint64 (progress, "start-time-extra-data")) / G_USEC_PER_SEC;
+      guint64 bytes_sec = (elapsed_time > 0) ? transferred_extra_data_bytes / elapsed_time : 0;
+      g_autofree char *formatted_bytes_transferred =
+        g_format_size_full (transferred_extra_data_bytes, 0);
+      g_autofree char *formatted_bytes_sec = NULL;
+
+      if (!bytes_sec) // Ignore first second
+        formatted_bytes_sec = g_strdup ("-");
+      else
+        formatted_bytes_sec = g_format_size (bytes_sec);
+
+      g_string_append_printf (buf, "Downloading extra data: %u%% (%lu/%lu) %s/s %s",
+                              (guint) ((((double) transferred_extra_data_bytes) / total_extra_data_bytes) * 100),
+                              transferred_extra_data_bytes, total_extra_data_bytes, formatted_bytes_sec, formatted_bytes_transferred);
+
+      /* Once we reach here, at least 55% of the data was fetched */
+      new_progress = 55 + (guint) (45 * ((double) transferred_extra_data_bytes) / total_extra_data_bytes);
+    }
   else if (outstanding_writes)
     {
       g_string_append_printf (buf, "Writing objects: %u", outstanding_writes);
+
+      /* Writing the objects advances 3% of progress */
+      new_progress = outstanding_extra_data ? 52 : 97;
+      new_progress += get_write_progress (outstanding_writes);
     }
   else
     {
@@ -1763,7 +1846,7 @@ flatpak_installation_fetch_remote_ref_sync (FlatpakInstallation *self,
     return flatpak_remote_ref_new (ref, checksum, remote_name);
 
   g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-               "Reference %s doesn't exist in remote\n", ref);
+               "Reference %s doesn't exist in remote", ref);
   return NULL;
 }
 
